﻿Imports System.IO
Imports System.IO.Compression
Imports System.Security.Cryptography


''' <summary>
''' Utilidades para la compresión y descompresión de datos
''' </summary>
''' <remarks></remarks>
Public NotInheritable Class CompressionUtilites

    ' Visibilidad privata para ocultar el constructor evitando poder crear un objeto de la clase 
    Private Sub New()
    End Sub

    ''' <summary>
    ''' Comprimir un byte()
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function ZipCompress(ByVal data As Byte()) As Byte()
        ' Compress the input array into the given memory stream.
        Dim stream As New MemoryStream()
        Using zip As New GZipStream(stream, CompressionMode.Compress, True)
            ' Write the input to the memory stream via the ZIP stream.
            zip.Write(data, 0, data.Length)
        End Using
        Return stream.ToArray
    End Function

    ''' <summary>
    ''' Descomprimir un byte()
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function ZipDeCompress(ByVal data As Byte()) As Byte()
        ' Create the zip stream to decompress.
        Using zip As New GZipStream(New MemoryStream(data), CompressionMode.Decompress, False)
            ' Read all bytes in the zip stream and return them.
            Return ReadAllBytes(zip)
        End Using
    End Function

    ''' <summary>
    ''' Comprimir un byte()
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function ZipCompress(ByVal data As String) As String
        Dim gData As Byte() = Convert.FromBase64String(data)
        ' Create the zip stream to decompress.
        Using zip As New GZipStream(New MemoryStream(gData), CompressionMode.Decompress, False)
            ' Read all bytes in the zip stream and return them.
            Return System.Text.Encoding.Unicode.GetString(ReadAllBytes(zip))
        End Using
    End Function

    ''' <summary>
    ''' Descomprimir un byte()
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function ZipDeCompress(ByVal data As String) As String
        Dim buffer As Byte() = System.Text.Encoding.Unicode.GetBytes(data)

        ' Create the zip stream to decompress.
        Using zip As New GZipStream(New MemoryStream(buffer), CompressionMode.Decompress, False)
            ' Read all bytes in the zip stream and return them.
            Return Convert.ToBase64String(ReadAllBytes(zip))
        End Using
    End Function



    ''' <summary>
    ''' Encrypts and compresses the input. Returns the compressed and encrypted content and the key and iv vector used.
    ''' </summary>
    ''' <param name="input">The input array.</param>
    ''' <param name="key">The key that has been used by the algorithm. (si no se indica se genera automáticamente)</param>
    ''' <param name="iv">The iv vector that has been used by the algorithm. (si no se indica se genera automáticamente)</param>
    ''' <returns></returns>
    Public Shared Function ZipEncryptAndCompress(input As Byte(), ByRef key As Byte(), ByRef iv As Byte()) As Byte()
        If input Is Nothing Then
            Throw New ArgumentNullException("Input")
        End If
        ' Compress the input array into the given memory stream.
        Dim stream As New MemoryStream()
        Using zip As New GZipStream(stream, CompressionMode.Compress, True)
            ' Write the input to the memory stream via the ZIP stream.
            zip.Write(input, 0, input.Length)
        End Using

        ' Create the keys and initalize the rijndael.
        Dim r As New RijndaelManaged()
        If key Is Nothing Then
            r.GenerateKey()
            key = r.Key
        Else
            r.Key = key
        End If
        If iv Is Nothing Then
            r.GenerateIV()
            iv = r.IV
        Else
            r.IV = iv
        End If

        ' Encrypt the compressed memory stream into the encrypted memory stream.
        Dim encrypted As New MemoryStream()
        Using cryptor As New CryptoStream(encrypted, r.CreateEncryptor(), CryptoStreamMode.Write)
            ' Write the stream to the encrypted memory stream.
            cryptor.Write(stream.ToArray(), 0, CInt(stream.Length))
            cryptor.FlushFinalBlock()
            ' Return the result.
            Return encrypted.ToArray()
        End Using
    End Function

    ''' <summary>
    ''' Decrypts and decompresses the input. Returns the decompressed and decrypted content.
    ''' </summary>
    ''' <param name="input">The input array.</param>
    ''' <param name="key">The key used for decrypt.</param>
    ''' <param name="iv">The iv vector used for decrypt.</param>
    ''' <returns></returns>
    Public Shared Function ZipDecryptAndDecompress(input As Byte(), key As Byte(), iv As Byte()) As Byte()
        If input Is Nothing Then
            Throw New ArgumentNullException("input")
        End If
        If key Is Nothing Then
            Throw New ArgumentNullException("key")
        End If
        If iv Is Nothing Then
            Throw New ArgumentNullException("iv")
        End If

        ' Initialize the rijndael
        Dim r As New RijndaelManaged()
        ' Create the array that holds the result.
        Dim decrypted As Byte() = New Byte(input.Length - 1) {}
        ' Create the crypto stream that is used for decrypt. The first argument holds the input as memory stream.
        Using decryptor As New CryptoStream(New MemoryStream(input), r.CreateDecryptor(key, iv), CryptoStreamMode.Read)
            ' Read the encrypted values into the decrypted stream. Decrypts the content.
            decryptor.Read(decrypted, 0, decrypted.Length)
        End Using

        ' Create the zip stream to decompress.
        Using zip As New GZipStream(New MemoryStream(decrypted), CompressionMode.Decompress, False)
            ' Read all bytes in the zip stream and return them.
            Return ReadAllBytes(zip)
        End Using
    End Function

    ''' <summary>
    ''' Reads all bytes in the given zip stream and returns them.
    ''' </summary>
    ''' <param name="zip">The zip stream that is processed.</param>
    ''' <returns></returns>
    Private Shared Function ReadAllBytes(zip As GZipStream) As Byte()
        If zip Is Nothing Then
            Throw New ArgumentNullException("zip")
        End If

        Dim buffersize As Integer = 100
        Dim buffer As Byte() = New Byte(buffersize - 1) {}
        Dim offset As Integer = 0, read As Integer = 0, size As Integer = 0
        Do
            ' If the buffer doesn’t offer enough space left create a new array
            ' with the double size. Copy the current buffer content to that array
            ' and use that as new buffer.
            If buffer.Length < size + buffersize Then
                Dim tmp As Byte() = New Byte(buffer.Length * 2 - 1) {}
                Array.Copy(buffer, tmp, buffer.Length)
                buffer = tmp
            End If

            ' Read the net chunk of data.
            read = zip.Read(buffer, offset, buffersize)

            ' Increment offset and read size.
            offset += buffersize
            size += read
        Loop While read = buffersize
        ' Terminate if we read less then the buffer size.
        ' Copy only that amount of data to the result that has actually been read!
        Dim result As Byte() = New Byte(size - 1) {}
        Array.Copy(buffer, result, size)
        Return result
    End Function

    Public Shared Function StringAsciiToBytes(ByVal value As String, Optional ByVal length As Integer = -1) As Byte()
        Dim encoding As System.Text.ASCIIEncoding = New System.Text.ASCIIEncoding()
        Dim b As Byte() = encoding.GetBytes(value)

        If length > 0 Then ReDim Preserve b(length)

        Return b
    End Function

    ''' <summary>
    ''' Codificación MD5, no tiene vuelta atras.
    ''' </summary>
    ''' <param name="value">Texto a codificar</param>
    ''' <returns>Array de bytes con la cadena codificada</returns>
    Public Shared Function MD5HashToByte(ByVal value As String) As Byte()

        If value Is Nothing Then
            Throw New ArgumentNullException("text", "Referencia nula no válida")
        End If

        Dim MD5 As New MD5CryptoServiceProvider
        Return MD5.ComputeHash(System.Text.UnicodeEncoding.UTF8.GetBytes(value))

    End Function

End Class
